from __future__ import with_statement
from __future__ import absolute_import
import signal
import sys
import getpass
import time
import threading
import requests
from requests.adapters import HTTPAdapter
import requests.packages.urllib3.util
import json
import signal
import os
import logging
log_file = os.path.join(os.path.expanduser('~'), "NDAValidationResults/debug_log_{}.txt").format(time.strftime("%Y%m%dT%H%M%S"))
logging.basicConfig(filename=log_file, level=logging.DEBUG, format="%(asctime)s:%(levelname)s:%(message)s")

if sys.version_info[0] < 3:
    import ConfigParser as configparser

    input = raw_input
    import thread
else:
    import configparser
    import _thread as thread

import xml.etree.ElementTree as ET

class Protocol(object):
    CSV = "csv"
    XML = "xml"
    JSON = "json"

    @staticmethod
    def get_protocol(cls):
        return cls.JSON

def api_request(api, verb, endpoint, data=None, session=None):

    retry = requests.packages.urllib3.util.retry.Retry(
        total=20,
        read=20,
        connect=20,
        backoff_factor=3,
        status_forcelist=(400, 403, 404, 500, 502, 504)
    )

    headers = {'accept': 'application/json'}
    auth = None
    if isinstance(api, Protocol):
        if api.get_protocol(api) == Protocol.CSV:
            headers.update({'content-type': 'text/csv'})
        elif api.get_protocol(api) == Protocol.XML:
            headers.update({'content-type': 'text/xml'})
    else:
        auth = requests.auth.HTTPBasicAuth(api.config.username, api.config.password)
        headers.update({'content-type': 'application/json'})

    if not session:
        session = requests.session()
        session.mount(endpoint, HTTPAdapter(max_retries=retry))
    r = None
    response = None
    try:
        r = session.send(requests.Request(verb, endpoint, headers, auth=auth, data=data).prepare(),
                         timeout=300, stream=False)

    except requests.exceptions.RequestException as e:
        print('\nAn error occurred while making {} request, check your endpoint configuration:\n'.
              format(e.request.method))
        logging.error(e)
        if api.__class__.__name__.endswith('Task'):
            api.shutdown_flag.set()
            thread.interrupt_main()
        exit_client(signal.SIGTERM)

    if r and r.ok:
        if api.__class__.__name__ == 'Download':
            return r, session
        else:
            try:
                response = json.loads(r.text)
            except ValueError:
                logging.error(ValueError)
                print('Your request returned an unexpected response, please check your endpoints.\n'
                      'Action: {}\n'
                      'Endpoint:{}\n'
                      'Status:{}\n'
                      'Reason:{}'.format(verb, endpoint, r.status_code, r.reason))
                if api.__class__.__name__.endswith('Task'):
                    api.shutdown_flag.set()
                    thread.interrupt_main()
                else:
                    raise Exception(ValueError)

    if r.status_code == 401:
        m = 'The NDA username or password is not recognized.'
        print(m)
        logging.error(m)
        r.raise_for_status()

    elif r.status_code == 400:
        response = json.loads(r.text)
        m = response['error'] + ': ' + response['message']
        print(m)
        logging.error(m)
        r.raise_for_status()

    elif r.status_code in (500, 502, 503, 504):
        response = r.text
        m = response
        print(m)
        logging.error(m)
        r.raise_for_status()

    return response, session

def exit_client(signal, frame=None, message=None):
    for t in threading.enumerate():
        try:
            t.shutdown_flag.set()
        except AttributeError as e:
            continue
    if message:
        print('\n\n{}'.format(message))
    else:
        print('\n\nExit signal received, shutting down...')
    print('Please contact NDAHelp@mail.nih.gov if you need assistance.')
    sys.exit(1)




import boto3
import botocore
import hashlib

from NDATools.Utils import *




class MultiPartsUpload:
    def __init__(self, bucket, prefix, config, access_key, secret_key, session_token):
        self.bucket = bucket
        self.prefix = prefix
        self.config = config
        self.access_key = access_key
        self.secret_key = secret_key
        self.session_token = session_token
        session = boto3.Session(aws_access_key_id=self.access_key,
                                aws_secret_access_key=self.secret_key,
                                aws_session_token=self.session_token,
                                region_name='us-east-1')
        self.client = session.client(service_name='s3')
        self.incomplete_mpu = []
        self.mpu_to_abort = {}

    def get_multipart_uploads(self):
        try:
            uploads = self.client.list_multipart_uploads(Bucket=self.bucket, Prefix=self.prefix)['Uploads']
            for u in uploads:
                if u not in self.incomplete_mpu:
                    self.incomplete_mpu.append(u)
                else:
                    self.mpu_to_abort[u['UploadId']] = u['Key']
        except KeyError:
            uploads = None

        if self.mpu_to_abort:
            self.abort_mpu()


    def abort_mpu(self):
        for upload_id, key in self.mpu_to_abort.items():
            self.client.abort_multipart_upload(
                Bucket=self.bucket, Key=key, UploadId=upload_id)
class Constants:
    PARTS = 'Parts'
    SIZE = 'Size'
    PART_NUM = 'PartNumber'
    ETAG = 'ETag'


class UploadMultiParts:
    def __init__(self, upload_obj, full_file_path, bucket, prefix, config, credentials):
        self.chunk_size = 8388608 #default
        self.upload_obj = upload_obj
        self.full_file_path = full_file_path
        self.upload_id = self.upload_obj['UploadId']
        self.bucket = bucket
        self.key = self.upload_obj['Key']
        filename = self.key.split(prefix+'/')
        filename = "".join(filename[1:])
        self.filename, self.file_size = self.full_file_path[filename]
        self.config = config
        self.access_key = credentials['access_key']
        self.secret_key = credentials['secret_key']
        self.session_token = credentials['session_token']
        session = boto3.Session(aws_access_key_id=self.access_key,
                                aws_secret_access_key=self.secret_key,
                                aws_session_token=self.session_token,
                                region_name='us-east-1')
        self.client = session.client(service_name='s3')
        self.completed_bytes = 0
        self.completed_parts = 0
        self.parts = []
        self.parts_completed = []


    def get_parts_information(self):
        self.upload_obj = self.client.list_parts(Bucket=self.bucket, Key=self.key,
                                             UploadId=self.upload_id)

        if Constants.PARTS in self.upload_obj:
            chunk_size = self.upload_obj[Constants.PARTS][0][Constants.SIZE] # size of first part should be size of all subsequent parts
            if chunk_size != 0:
                self.chunk_size = chunk_size
            for p in self.upload_obj[Constants.PARTS]:
                try:
                    self.parts.append({Constants.PART_NUM: p[Constants.PART_NUM], Constants.ETAG: p[Constants.ETAG]})
                    self.parts_completed.append(p[Constants.PART_NUM])
                except KeyError:
                    pass

        self.completed_bytes = self.chunk_size * len(self.parts)


    def check_md5(self, part, data):

        ETag = (part[Constants.ETAG]).split('"')[1]

        md5 = hashlib.md5(data).hexdigest()
        if md5 != ETag:
            message = "The file seems to be modified since previous upload attempt(md5 value does not match)."
            exit_client(signal=signal.SIGTERM, message=message) # force exit because file has been modified (data integrity)

    def upload_part(self, data, i):
        part = self.client.upload_part(Body=data, Bucket=self.bucket, Key=self.key, UploadId=self.upload_id, PartNumber=i)
        self.parts.append({Constants.PART_NUM: i, Constants.ETAG: part[Constants.ETAG]})
        self.completed_bytes += len(data)


    def complete(self):
        self.client.complete_multipart_upload(
            Bucket=self.bucket,
            Key=self.key,
            UploadId=self.upload_id,
            MultipartUpload={Constants.PARTS: self.parts})


from __future__ import with_statement
from __future__ import absolute_import
import sys
import requests.packages.urllib3.util
from tqdm import tqdm
import boto3
import botocore
import signal

if sys.version_info[0] < 3:
    input = raw_input

from NDATools.Configuration import *
from NDATools.Utils import *


class SubmissionPackage:
    def __init__(self, uuid, associated_files, config, username=None, password=None, collection=None, title=None,
                 description=None, alternate_location=None, allow_exit=False):

        self.config = config
        self.aws_access_key = self.config.aws_access_key
        self.aws_secret_key = self.config.aws_secret_key
        self.api = self.config.submission_package_api
        self.validationtool_api = self.config.validationtool_api
        self.uuid = uuid
        self.associated_files = associated_files
        self.full_file_path = {}
        self.username = self.config.username
        self.password = self.config.password
        self.no_match = []
        if username:
            self.config.username = username
        if password:
            self.config.password = password
        if title:
            self.config.title = title
        if description:
            self.config.description = description
        self.username = self.config.username
        self.password = self.config.password
        self.dataset_name = self.config.title
        self.dataset_description = self.config.description
        self.package_info = {}
        self.download_links = []
        self.package_id = None
        self.package_folder = None
        self.collection_id = collection
        self.endpoint_title = alternate_location
        self.collections = {}
        self.endpoints = []
        self.get_collections()
        self.get_custom_endpoints()
        self.validation_results = []
        self.no_read_access = set()
        self.submission_packages_dir = os.path.join(os.path.expanduser('~'), self.config.submission_packages)
        if not os.path.exists(self.submission_packages_dir):
            os.mkdir(self.submission_packages_dir)
        self.exit = allow_exit

    def get_collections(self):
        collections, session = api_request(self,
                                           "GET",
                                           "/".join([self.validationtool_api, "user/collection"]))
        if collections:
            for c in collections:
                self.collections.update({c['id']: c['title']})

    def get_custom_endpoints(self):
        endpoints, session = api_request(self,
                                         "GET",
                                         "/".join([self.validationtool_api, "user/customEndpoints"]))
        if endpoints:
            for endpoint in endpoints:
                self.endpoints.append(endpoint['title'])

    def set_upload_destination(self, hide_input=True):
        if len(self.collections) > 0 or len(self.endpoints) > 0:
            user_input = None
            if self.config.collection_id and self.config.endpoint_title:
                print('You selected both a collection and an alternate endpoint.\n'
                      'These options are mutually exclusive, please specify only one.')
            elif self.config.endpoint_title:
                self.endpoint_title = self.config.endpoint_title
            elif self.config.collection_id:
                self.collection_id = self.config.collection_id
            else:
                if not hide_input:
                    user_input = input('\nEnter -c <collection ID> OR -a <alternate endpoint>:')
            while True:
                if user_input:
                    try:
                        if user_input.startswith('-c'):
                            collection_id = user_input.split(" ")
                            self.collection_id = collection_id[1]
                            self.endpoint_title = ""
                        elif user_input.startswith('-a'):
                            endpoint_title = user_input.split(" ")
                            self.endpoint_title = endpoint_title[1]
                            self.collection_id = ""
                    except IndexError:
                        print('Do not leave this section blank.')
                try:
                    if self.collection_id and int(self.collection_id) in self.collections:
                        break
                    elif self.endpoint_title in self.endpoints:
                        break
                    else:
                        if not hide_input:
                            print ('Collection IDs:')
                            for k, v in self.collections.items():
                                print('{}: {}'.format(k, v.encode('ascii', 'ignore')))
                            print('\nAlternate Endpoints:')
                            for endpoint in self.endpoints:
                                print(endpoint.encode('ascii', 'ignore'))
                            message = '\nYou do not have access to submit to the collection or alternate upload location: {} '.format(self.collection_id or self.endpoint_title)
                            print(message)
                            user_input = input('\nEnter -c <collection ID> OR -a <alternate endpoint> from the list above:')
                        else:
                            message = 'Incorrect/Missing collection ID or alternate endpoint.'
                            if self.exit:
                                exit_client(signal=signal.SIGTERM,
                                            message=message)
                            else:
                                raise Exception(message)
                except (AttributeError, ValueError, TypeError) as e:
                    message = 'Error: Input must start with either a -c or -a and be an integer or string value, respectively.'
                    if not hide_input:
                        print(message)
                        user_input = input('\nEnter -c <collection ID> OR -a <alternate endpoint>:')
                    else:
                        message = 'Incorrect/Missing collection ID or alternate endpoint.'
                        if self.exit:
                            exit_client(signal=signal.SIGTERM,
                                            message=message)
                        else:
                            raise Exception(message)
        else:
            message = 'The user {} does not have permission to submit to any collections or alternate upload locations.'.format(self.config.username)
            if self.exit:
                exit_client(signal=signal.SIGTERM,
                        message=message)
            else:
                raise Exception(message)

    def recollect_file_search_info(self):
        retry = input('Press the "Enter" key to specify directory/directories OR an s3 location by entering -s3 '
                      '<bucket name> to locate your associated files:')
        response = retry.split(' ')
        self.directory_list = response
        if response[0] == '-s3':
            self.source_bucket = response[1]
            self.source_prefix = input('Enter any prefix for your S3 object, or hit "Enter": ')
            if self.source_prefix == "":
                self.source_prefix = None
            if self.aws_access_key == "":
                self.aws_access_key = input('Enter the access_key for your AWS account: ')
            if self.aws_secret_key == "":
                self.aws_secret_key = input('Enter the secret_key for your AWS account: ')


    def check_read_permissions(self, file):
        try:
            open(file)
            return True
        except IOError:
            return False
        except PermissionError:
            return False

    def file_search(self, directories=None, source_bucket=None, source_prefix=None, access_key=None, secret_key=None, retry_allowed=False):
        def raise_error(error, l = []):
            m = '\n'.join([error] + list(set(l)))
            raise Exception(m)

        self.directory_list = directories
        self.source_bucket = source_bucket
        self.source_prefix = source_prefix
        self.aws_access_key = access_key
        self.aws_secret_key = secret_key

        if not self.directory_list and not self.source_bucket:
            if retry_allowed:
                self.recollect_file_search_info()
            else:
                error ='Missing directory and/or an S3 bucket.'
                raise_error(error)

        if not self.no_match:
            for a in self.associated_files:
                for file in a:
                    self.no_match.append(file)

        # local files
        if self.directory_list:
            for file in self.no_match[:]:
                if file.startswith('/'):
                    f = file[1:]
                else:
                    f = file
                for d in self.directory_list:
                    file_name = os.path.join(d, f)
                    if os.path.isfile(file_name):
                        if not self.check_read_permissions(file_name):
                            self.no_read_access.add(file_name)
                        self.full_file_path[file] = (file_name, os.path.getsize(file_name))
                        self.no_match.remove(file)
                        break

        # files in s3
        no_access_buckets = []
        if self.source_bucket:
            if self.aws_access_key is "":
                self.aws_access_key = input('Enter the access_key for your AWS account: ')
            if self.aws_secret_key is "":
                self.aws_secret_key = input('Enter the secret_key for your AWS account: ')
            s3 = boto3.session.Session(aws_access_key_id=self.aws_access_key, aws_secret_access_key=self.aws_secret_key)
            s3_client = s3.client('s3')
            for file in self.no_match[:]:
                key = file
                if self.source_prefix:
                    key = '/'.join([self.source_prefix, file])
                file_name = '/'.join(['s3:/', self.source_bucket, key])
                try:
                    response = s3_client.head_object(Bucket=self.source_bucket, Key=key)
                    self.full_file_path[file] = (file_name, int(response['ContentLength']))
                    self.no_match.remove(file)
                except botocore.exceptions.ClientError as e:
                    # If a client error is thrown, then check that it was a 404 error.
                    # If it was a 404 error, then the bucket does not exist.
                    error_code = int(e.response['Error']['Code'])
                    if error_code == 404:
                        pass
                    if error_code == 403:
                        no_access_buckets.append(self.source_bucket)
                        pass

        if self.no_match:
            if no_access_buckets:
                message = 'Your user does NOT have access to the following buckets. Please review the bucket ' \
                          'and/or your AWS credentials and try again.'
                if retry_allowed:
                    print('\n', message)
                    for b in no_access_buckets:
                        print(b)
                else:
                    error = "".join(['Bucket Access:', message])
                    raise_error(error, no_access_buckets)
            message = 'You must make sure all associated files listed in your validation file' \
                      ' are located in the specified directory or AWS bucket. Associated file not found in specified directory:\n'
            if retry_allowed:
                print('\n', message)
                for file in self.no_match:
                    print(file)
                self.recollect_file_search_info()
                self.file_search(self.directory_list, self.source_bucket, self.source_prefix, self.aws_access_key,self.aws_secret_key,
                                 retry_allowed=True)
            else:
                error = "".join(['Missing Files:', message])
                raise_error(error, self.no_match)

        while self.no_read_access:
            message = 'You must make sure you have read-access to all the of the associated files listed in your validation file. ' \
                      'Please update your permissions for the following associated files:\n'
            if retry_allowed:
                print(message)
                for file in self.no_read_access:
                    print(file)
                self.recollect_file_search_info()
                [self.no_read_access.remove(i) for i in [file for file in self.no_read_access if self.check_read_permissions(file)]]
            else:
                error = "".join(['Read Permission Error:', message])
                raise_error(error, self.no_match)

        self.config.directory_list = self.directory_list
        self.config.source_bucket = self.source_bucket
        self.config.source_prefix = self.source_prefix
        self.config.aws_access_key = self.aws_access_key
        self.config.aws_secret_key = self.aws_secret_key


    def build_package(self):
        def raise_error(value):
            raise Exception("Missing {}. Please try again.".format(value))


        if self.dataset_name is None:
            raise_error('dataset name')

        if self.dataset_description is None:
            raise_error('dataset description')

        if self.collection_id is None and self.endpoint_title is None:
            raise_error('collection ID or alternate endpoint')

        self.package_info = {
            "package_info": {
                "dataset_description": self.dataset_description,
                "dataset_name": self.dataset_name,
                "collection_id": self.collection_id,
                "endpoint_title": self.endpoint_title
            },
            "validation_results":
                self.uuid
        }

        json_data = json.dumps(self.package_info)
        response, session = api_request(self, "POST", self.api, json_data)
        if response:
            try:
                self.package_id = response['submission_package_uuid']
                for r in response['validation_results']:
                    self.validation_results.append(r['id'])
                self.submission_package_uuid = str(response['submission_package_uuid'])
                self.create_date = str(response['created_date'])
                self.expiration_date = str(response['expiration_date'])
            except KeyError:
                message = 'There was an error creating your package.'
                if response['status'] == Status.ERROR:
                    message = response['errors'][0]['message']
                if self.exit:
                    exit_client(signal.SIGTERM, message=message)
                else:
                    raise Exception(message)

            polling = 0
            while response['package_info']['status'] == Status.PROCESSING:
                response, session = api_request(self, "GET", "/".join([self.api, self.package_id]), session=session)
                polling += 1
                self.package_id = response['submission_package_uuid']
            if response['package_info']['status'] == Status.COMPLETE:
                for f in [f for f in response['files']
                          if f['type'] in ('Submission Memento', 'Submission Data Package')]:
                    for key, value in f['_links'].items():
                        for k, v in value.items():
                            self.download_links.append((v, "/".join(f['path'].split('/')[4:])))
            else:
                message = 'There was an error in building your package.'
                if response['package_info']['status'] == Status.SYSERROR:
                    message=response['errors']['system'][0]['message']
                if self.exit:
                    exit_client(signal.SIGTERM, message=message)
                else:
                    raise Exception(message)
        else:
            message='There was an error with your package request.'
            if self.exit:
                exit_client(signal.SIGTERM, message=message)
            else:
                raise Exception(message)

    def download_package(self, hide_progress):
        folder = self.download_links[0][1]
        folder = folder.split('/')
        self.package_folder = folder[0]
        session = requests.session()
        total_package_size = 0
        for i, (url, file_name) in enumerate(self.download_links):
            r = session.get(url, auth=(self.username, self.password), stream=True)
            size = r.headers['content-length']
            total_package_size += int(size)
            self.download_links[i] = (url, file_name, int(size))
            r.close()
        if not hide_progress:
            package_download = tqdm(total=total_package_size,
                                unit="bytes",
                                unit_scale=True,
                                desc="Submission Package Download",
                                ascii=os.name == 'nt')
        for url, file_name, size in self.download_links:
            new_path = os.path.join(os.path.expanduser('~'), self.config.submission_packages)
            path = os.path.join(new_path, self.package_folder)
            if not os.path.exists(path):
                os.mkdir(path)
            file_name = file_name.split('/')
            file_name = file_name[1]
            file = os.path.join(path, file_name)
            r = session.get(url, auth=(self.username, self.password), stream=True)
            with open(file, 'wb') as out_file:
                for chunk in r.iter_content(chunk_size=1024 * 1024):
                    if chunk:
                        out_file.write(chunk)
                        if not hide_progress:
                            package_download.update(sys.getsizeof(chunk))
            session.close()
        if not hide_progress:
            if package_download.total > package_download.n:
                package_download.update(package_download.total - package_download.n)
            package_download.close()


class Status:
    UPLOADING = 'Uploading'
    SYSERROR = 'SystemError'
    COMPLETE = 'complete'
    ERROR = 'error'
    PROCESSING = 'processing'


from __future__ import with_statement
from __future__ import absolute_import
import sys
if sys.version_info[0] < 3:
    input = raw_input
import argparse
from NDATools.Download import Download
from NDATools.Configuration import *


def parse_args():
    parser = argparse.ArgumentParser(
        description='This application allows you to enter a list of aws S3 paths and will download the files to your local drive '
                    'in your home folder. Alternatively, you may enter a packageID,an NDA data structure file or a text file with s3 links, '
                    'and the client will download all associated files from S3 listed.',
        usage='%(prog)s <S3_path_list>')

    parser.add_argument('paths', metavar='<S3_path_list>', type=str, nargs='+', action='store',
                        help='Will download all S3 files to your local drive')

    parser.add_argument('-dp', '--package', action='store_true',
                        help='Flags to download all S3 files in package.')

    parser.add_argument('-t', '--txt', action='store_true',
                        help='Flags that a text file has been entered from where to download S3 files.')

    parser.add_argument('-ds', '--datastructure', action='store_true',
                        help='Flags that a  data structure text file has been entered from where to download S3 files.')

    parser.add_argument('-u', '--username', metavar='<arg>', type=str, action='store',
                        help='NDA username')

    parser.add_argument('-p', '--password', metavar='<arg>', type=str, action='store',
                        help='NDA password')

    parser.add_argument('-r', '--resume', metavar='<arg>', type=str, nargs=1, action='store',
                        help='Flags to restart a download process. If you already have some files downloaded, you must enter the directory where they are saved.')


    parser.add_argument('-d', '--directory', metavar='<arg>', type=str, nargs=1, action='store',
                        help='Enter an alternate full directory path where you would like your files to be saved.')

    parser.add_argument('-wt', '--workerThreads', metavar='<arg>', type=int, action='store',
                        help='Number of worker threads')

    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Option to print out more detailed messages as the program runs.')
    args = parser.parse_args()

    return args



def configure(username, password):
    if os.path.isfile(os.path.join(os.path.expanduser('~'), '.NDATools/settings.cfg')):
        config = ClientConfiguration(os.path.join(os.path.expanduser('~'), '.NDATools/settings.cfg'), username, password)
    else:
        config = ClientConfiguration('clientscripts/config/settings.cfg', username, password)
        config.read_user_credentials()
        config.make_config()
    return config

def main():
    args = parse_args()
    config = configure(args.username, args.password)

    # directory where files will be downloaded
    if args.directory:
        dir = args.directory[0]
    else:
        dir = os.path.join(os.path.expanduser('~'), 'AWS_downloads')

    # determine which method to use to collect s3 file locations
    if args.package:
        links = 'package'
        # this option only downloads data structure files from the package.
        # after data structure files have been downloaded, it will begin downloading associated files listed in each
        # structure
    elif args.txt:
        links = 'text'
    elif args.datastructure:
        links = 'datastructure'
    else:
        links = 'paths'

    # if some files were already downloaded, resume option will only download new files
    resume = False
    prev_directory = None
    if args.resume:
        resume = True
        prev_directory = args.resume[0]

    verbose = False
    if args.verbose:
        verbose = True

    s3Download = Download(dir, config, verbose=verbose)
    s3Download.get_links(links, args.paths, filters=None)
    if len(s3Download.path_list) == 0:
        return
    s3Download.get_tokens()
    s3Download.start_workers(resume, prev_directory, args.workerThreads)

    # download associated files from package
    #if args.package:
    #    s3Download.searchForDataStructure(resume, prev_directory)


    if verbose:
        print('Finished downloading all files.')

if __name__ == "__main__":
    main()


#!/usr/bin/env python

from setuptools import find_packages, setup


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
        name='nda_tools',
        description="NIMH Data Archive Python Client",
        long_description=long_description,
        long_description_content_type="text/markdown",
        install_requires=['boto3>=1.4.7', 'botocore>=1.7.48', 'tqdm','requests'],
        version='0.1.18',
        author='NDA',
        author_email='NDAHelp@mail.nih.gov',
        url="https://github.com/NDAR/nda-tools/tree/master/NDATools",
        license='MIT',
        packages=find_packages(),
        include_package_data=True,
        data_files=[('config', ['NDATools/clientscripts/config/settings.cfg'])],
        entry_points={
            'console_scripts': [
                'vtcmd = NDATools.clientscripts.vtcmd:main',
                'downloadcmd = NDATools.clientscripts.downloadcmd:main']
        }
    )
